﻿#!/usr/bin/python
#-*- coding: utf-8 -*-

# SRC4392 ASRC module for RAL-Raspberry pi
#         V.1.0  29-Sept.-2017 for Python 2.7
#           1.1  06-Nov. -2017
#                C.okamura / RATOC Systems, Inc.

import sys
import smbus
import time
#import math

bus = smbus.SMBus(1)        # i2c bus number : 1
devadrs = 0x70              # i2c default address of SRC4392 a6...a0


#  SRC4392  Register Page 0

# Power down and Reset control register
PDWN_RST_CTR      = 0x01
RESET_SRC         = 0x80     #  b7, 1: force RESET, all control registers are reset to default state.
RELEASE_RESET_SRC = 0x00     #      0:(default): disable RESET, normal operation.
SET_PDALL         = 0x00     #  b5, 0:(default): force all SRC function to Power Down mode.
RELEASE_PDALL     = 0x20     #      1: enable all SRC function execute normal operation.
RELEASE_PDPA      = 0x10     #  Power-Down conterol for Serial I2S port A, 0:(default) power-down state, 1: enable Port-A.
RELEASE_PDPB      = 0x08     #  Power-Down conterol for Serial I2S port B, 0:(default) power-down state, 1: enable Port-B.
RELEASE_PDTX      = 0x04     #  Power-Down conterol for S/PDIF transmitter, 0:(default) power-down state, 1: enable Transmitter.
RELEASE_PDRX      = 0x02     #  Power-Down conterol for S/PDIF receivers, 0:(default) power-down state, 1: enable Receivers.
RELEASE_PDSRC     = 0x01     #  Power-Down conterol for Sample Rate Converter, 0:(default) power-down state, 1: enable SRC.

# Global Interrupt Status register ( RO )
#define GINT_STR            0x02
#define     TX_INT          0x04          // 1: INT request from Transmitter.
#define     RX_INT          0x02          // 1: INT request from Receiver.
#define     SRC_INT         0x01          // 1: INT request from SRC

# I2S Port A control register #1
I2S_PORTA_CTR1  =      0x03
I2SPA_MUTE      =  0x40          # 1: force Port A SDOUT 'Low',  0:(default) normal operation.
I2SPA_IS_LPBACK =        0x00    # (default)  input source through PortA is routed to PortA output(loop back).
I2SPA_IS_PB     =        0x10    # input source through PortB is routed to PortA output.
I2SPA_IS_DIR    =        0x20    # input source through DIR is routed to PortA output.
I2SPA_IS_SRC    =        0x30    # input source from SRC is routed to PortA output.
I2SPA_SLV_MODE  =        0x00    # b3, PortA works as a Slave mode ( use external LPCLK and BCLK): default.
I2SPA_MST_MODE  =        0x08    # Port A works as a Master mode ( provide LRCLK and BCLK to DAC )
I2SPA_FMT_24LJ  =        0x00    # Port A uses 24bit Left justified format(default).
I2SPA_FMT_I2S_P24 =      0x01    # Port A uses 24bit Phillips I2S format.
I2SPA_FMT_16RJ  =        0x04    # Port A uses 16bit Right justified format.
I2SPA_FMT_18RJ  =        0x05    # Port A uses 18bit Right justified format.
I2SPA_FMT_20RJ  =        0x06    # Port A uses 20bit Right justified format.
I2SPA_FMT_24RJ  =        0x07    # Port A uses 24bit Right justified format.

# I2S Port A control register #2
I2S_PORTA_CTR2  =    0x04
I2SPA_MCKS_MCLK =    0x00        # Port A use MCLKI(25) as a Master Clock.
I2SPA_MCKS_RXCKI =   0x04        # Port A use RXCKI(13) as a Master Clock.
I2SPA_MCKS_RXCKO =   0x08        # Port A use RXCKO(Extracted Reference Clock from input source) as a Master Clock.
I2SPA_MCLK_128FS =   0x00        # MCLK = 128 * fs ( default ).
I2SPA_MCLK_256FS =   0x01        # MCLK = 256 * fs ( default ).
I2SPA_MCLK_384FS =   0x02        # MCLK = 384 * fs ( default ).
I2SPA_MCLK_512FS =   0x03        # MCLK = 512 * fs ( default ).


# I2S Port B control register #1
I2S_PORTB_CTR1   =  0x05
I2SPB_MUTE       =  0x40         # 1: force Port B SDOUT 'Low',  0:(default) normal operation.
I2SPB_IS_LPBACK  =  0x00         # (default)  input source through PortB is routed to PortB output(loop back).
I2SPB_IS_PA      =  0x10         # input source through PortA is routed to PortB output.
I2SPB_IS_DIR     =  0x20         # input source through DIR is routed to PortB output.
I2SPB_IS_SRC     =  0x30         # input source from SRC is routed to PortB output.
I2SPB_SLV_MODE   =  0x00         # b3, PortB works as a Slave mode ( use external LPCLK and BCLK): default.
I2SPB_MST_MODE   =  0x08         # Port B works as a Master mode ( provide LRCLK and BCLK to DAC )
I2SPB_FMT_24LJ   =  0x00         # Port B uses 24bit Left justified format(default).
I2SPB_FMT_I2S_P24 = 0x01         # Port B uses 24bit Phillips I2S format.
I2SPB_FMT_16RJ   =  0x04         # Port B uses 16bit Right justified format.
I2SPB_FMT_18RJ   =  0x05         # Port B uses 18bit Right justified format.
I2SPB_FMT_20RJ   =  0x06         # Port B uses 20bit Right justified format.
I2SPB_FMT_24RJ   =  0x07         # Port B uses 24bit Right justified format.

# I2S Port B control register #2
I2S_PORTB_CTR2   =  0x06
I2SPB_MCKS_MCLK  =  0x00         # Port B use MCLKI(25) as a Master Clock.
I2SPB_MCKS_RXCKI =  0x04         # Port B use RXCKI(13) as a Master Clock.
I2SPB_MCKS_RXCKO =  0x08         # Port B use RXCKO(Recovered Master Clock from input source) as a Master Clock.
I2SPB_MCLK_128FS =  0x00         # MCLK = 128 * fs ( default ).
I2SPB_MCLK_256FS =  0x01         # MCLK = 256 * fs ( default ).
I2SPB_MCLK_384FS =  0x02         # MCLK = 384 * fs ( default ).
I2SPB_MCLK_512FS =  0x03         # MCLK = 512 * fs ( default ).

# Transmitter Control Register #1
DIT_CTR1       =    0x07
DIT_TXCLK_MCLK =    0x00          # DIT uses MCLKI as Master Clock( default )
DIT_TXCLK_RCLK =    0x80          # DIT uses Recoverd Master Clock from Input source as a Master Clock
DIT_MCLK_128FS =    0x00          # MCLK = 128 * fs ( default ).
DIT_MCLK_256FS =    0x20          # MCLK = 256 * fs ( default ).
DIT_MCLK_384FS =    0x40          # MCLK = 384 * fs ( default ).
DIT_MCLK_512FS =    0x60          # MCLK = 512 * fs ( default ).
DIT_IS_PA      =    0x00          # input source is Port A (default).
DIT_IS_PB      =    0x08          # input source is Port B
DIT_IS_DIR     =    0x10          # input source is DIR
DIT_IS_SRC     =    0x18          # input source is SRC
DIT_BLSM_IN    =    0x00          # Block Start input mode ( default )
DIT_BLSM_OUT   =    0x04          # Block Start output mode     
DIT_VALID_ON   =    0x00          # indicate Valid data (default)
DIT_VALID_OFF  =    0x02          # indicate Transmitted data is invalid.
DIT_BSSL_DSLIP =    0x00          # TSLIP interrupt will be triggerd with Data Slip Condition ( default )
DIT_BSSL_BSC   =    0x01          # TSLIP interrupt will be triggerd with Block Start condition.

# Transmitter Control Register #2
DIT_CTR2       =    0x08
DIT_TXLD_OFF   =    0x01          # Transmitter Line Driver OFF ( Send 'L' )
DIT_TXLD_ON    =    0x00          # Transmitter Line Driver ON  ( default )
DIT_TXMUTE_ON  =    0x02          # Transmitter Output OFF ( Zero data )
DIT_TXMUTE_OFF =    0x00          # Normal operation ( default )
DIT_AESOUT_ON  =    0x00          # AES output driver ON, normal operation ( default )
DIT_AESOUT_OFF =    0x04          # AES output driver OFF ( Send 'L' ).
DIT_TXBTD_EN   =    0x00          # User and Transmitter Access Buffer enabled
DIT_TXBTD_DA   =    0x08          # User and Transmitter Access buffer disabled.
DIT_TXLDMUX_ENC =   0x00          # TX Line Driver input source is DIT AES3 Encorder ( default)
DIT_TXLDMUX_BYP =   0x10          # TX Line Driver skip Multiplexer.
DIT_AESMUX_ENC  =   0x00          # AES output is routed from AES3 Encorder. ( default )
DIT_AESMUX_BYP  =   0x10          # AES output skip Multiplexer.
DIT_BYPMUX_RX1  =   0x00          # Selext RX1 for Line receiver out put as Bypass Multiplexer input (default )
DIT_BYPMUX_RX2  =   0x40          # Select RX2 for Line receiver out put as Bypass Multiplexer input.
DIT_BYPMUX_RX3  =   0x80          # Selext RX3 for Line receiver out put as Bypass Multiplexer input (default )
DIT_BYPMUX_RX4  =   0xc0          # Select RX4 for Line receiver out put as Bypass Multiplexer input.

# Transmitter Control Register #3
DIT_CTR3        =   0x09
DIT_TXCUSTS_NOUP =  0x00          # Channel Status and User data buffer are not updated ( default )
DIT_TXCUSTS_HIF  =  0x01          # Channel Status and User data buffer are updated through Host Interface(I2C or SPI)
DIT_TXCUSTS_DIRRA = 0x02          # Channel Status and User data buffer are updated through DIR RA buffers
DIT_TXCUSTS_HIFRA = 0x03          # Channel Status and User data buffer are updated through both Host Interface(I2C or SPI) and DIR RA buffer.
DIT_VALSEL_R07    = 0x00          # Validity bit is same as DIT_CTR1 b1.
DIT_VALSEL_DIR    = 0x04          # Validity bit is updated by DIR buffer

#  SRC and DIT status register
#define SRC_DIT_STSR     0x0a
#define    SRC_DIT_STS_TBTI     0x01    // TXBTD Interrupt
#define    SRC_DIT_STS_TSLIP    0x02    // TSLIP interrupt
#define    SRC_DIT_STS_READY    0x10    // SRC Rate Estimator Ready Status
#define    SRC_DIT_STS_RATIO    0x20    // SRC Ratio Status

#  SRC and DIT Interrupt Mask register
#define SRC_DIT_IMSKR    0x0b
#define    SRC_DIT_EN_TBTI      0x01    // Enable TXBTD Interrupt
#define    SRC_DIT_EN_TSLIP     0x02    // Enable TSLIP interrupt
#define    SRC_DIT_EN_READY     0x10    // Enable SRC Rate Estimator Ready Status
#define    SRC_DIT_EN_RATIO     0x20    // Enable SRC Ratio Status

#  SRC and DIT Interrupt Mode register
#define SRC_DIT_IMODER  0x0c
#define   SRC_DIT_TBTI_MODE_RE  0x00    // TBT Interrupt is set at Rising Edge (default)
#define   SRC_DIT_TBTI_MODE_FE  0x01    // TBT Interrupt is set at Falling Edge
#define   SRC_DIT_TBTI_MODE_LA  0x02    // TBT Interrupt is set at Level Sense
#define   SRC_DIT_TSLPI_MODE_RE 0x00    // TSLIP Interrupt is set at Rising Edge (default)
#define   SRC_DIT_TSLPI_MODE_FE 0x04    // TSLIP Interrupt is set at Falling Edge
#define   SRC_DIT_TSLPI_MODE_LA 0x08    // TSLIP Interrupt is set at Level Sense
#define   SRC_DIT_TRDYI_MODE_RE 0x00    // READY Interrupt is set at Rising Edge (default)
#define   SRC_DIT_TRDYI_MODE_FE 0x10    // READY Interrupt is set at Falling Edge
#define   SRC_DIT_TRDYI_MODE_LA 0x20    // READY Interrupt is set at Level Sense
#define   SRC_DIT_TRTOI_MODE_RE 0x00    // RETIO Interrupt is set at Rising Edge (default)
#define   SRC_DIT_TRTOI_MODE_FE 0x40    // RETIO Interrupt is set at Falling Edge
#define   SRC_DIT_TRTOI_MODE_LA 0x80    // RETIO Interrupt is set at Level Sense

#  Receiver Control Register #1
#define DIR_CTR1       0x0d
#define   DIR_COAX_SEL      0x00     // RX1 select(default), COAX input ( SV_8741 ).
#define   DIR_TOSLINK_SEL   0x01     // RX2 select, TOSLINK input ( SV_8741 ).
#define   DIR_iPod_SEL      0x02     // RX3 select. iPod Digital Link S/PDIF output ( SV_8741 ).
#define   DIR_NW_SEL        0x03     // RX4 select. Network receiver S/PDIF output ( SV_8741 ).
#define   DIR_REFCLK_RXCKI  0x00     // Reference Clock source is get through RXCKI pin.
#define   DIR_REFCLK_MCLK   0x08     // Reference Clock source is get through MCLK pin.
#define   DIR_CUBUF_TRON    0x00     // C and U buffer transfer to DIT
#define   DIR_CUBUF_TROFF   0x10     // C and U buffer is not transfered to DIT, must check DIR buffer.

#  Receiver Control Register #2
#define DIR_CTR2      0x0e
#define   DIR_RXCKO_HZ      0x00     // disable RXCK output, set it Hi-Z mode ( default )
#define   DIR_RXCKO_EN      0x01     // Enable RXCK output, recovered MCLK is available at the pin.
#define   DIR_RXCKO_DV1     0x00     // Passthrough RXCK output, no division.
#define   DIR_RXCKO_DV2     0x02     // RXCK is devided by 2.
#define   DIR_RXCKO_DV4     0x04     // RXCK is devided by 4.
#define   DIR_RXCKO_DV8     0x06     // RXCK is devided by 8.
#define   DIR_AMUTE_LL      0x08     // force Receiver output to MUTE automaticaly at Loss of Lock state.
#define   DIR_PLL_FREERUN   0x10     // PLL2 free run at loss of lock state.

#  Receiver PLL1 Configuration register #1 to #3
#  D-9  Ref.CLK = 22.5792 MHz --> P=2, J =8, D=7075 ( error = 0.0002% )
#    P = 0b0010, J = 0b001000, D = 0x2c3 = 0b0001101110100011
#define DIR_PLL1_CFGR1  0x0f        //  0x22
#define DIR_PLL1_CFGR2  0x10        //  0x1b
#define DIR_PLL1_CFGR3  0x11        //  0xa3

#  Receiver Non-PCM Audio Stream Detection Status Register (RO)
#define DIR_AS_NPCM_STR  0x12
#define   DETECT_IEC61937   0x01     // detect IEC61937 ( Dolby 5.1 ) stream.
#define   DETECT_DTS        0x02     // detect DTS CD/LD stream

#  Receiver Status Register 1: Recoverd Clock Rate (RO)
#define DIR_RCVCLK_RATER  0x13
#define   RCVCLK_NOTDETERMINED   0x00  // Recovered Clock Rate is not determined
#define   RCVCLK_128FS           0x01  // Recovered Clock Rate is 128fs
#define   RCVCLK_256fs           0x02  // Recovered Clock Rate is 256fs
#define   RCVCLK_512fs           0x03  // Recovered Clock Rate is 512fs.

#  Receiver Status Register 2 (RO)
#define DIR_STATUSR2  0x14
#define   DIR_STS_CSCRCER       0x80  // Cahnnel Status CRC Error detected
#define   DIR_STS_PARITYER      0x40  // Parity Error detected
#define   DIR_STS_NONVALID      0x20  // Non-Valid Audio data detected
#define   DIR_STS_BPERR         0x10  // Bipolar Encoding Error detected
#define   DIR_STS_QCHG          0x08  // Q-channel data has changed.
#define   DIR_STS_UNLOCK        0x04  // DIR Lock error, AES3 decorder and PLL2 unlocked detected
#define   DIR_STS_QCRC          0x02  // Q-Channel sub code CRC error detected
#define   DIR_STS_RBTI          0x01  // DIR RA buffer transfer completed

#  Receiver Status Register 3 (RO)
#define DIR_STATUSR3  0x15
#define   DIR_STS_OSLIP        0x01  // DIR Output Data Slip/repeat Error detected

#  Receiver Interrupt Mask(Enable) Register #1
#define DIR_INTMASKR1  0x16
#define   DIR_CSCRCER_IEN       0x80  // enable Cahnnel Status CRC Error Interupt
#define   DIR_PARITYER_IEN      0x40  // enable Parity Error Interupt
#define   DIR_NONVALID_IEN      0x20  // enable Non-Valid Audio data detected Interrupt
#define   DIR_BPERR_IEN         0x10  // enable Bipolar Encoding Error detected Interrupt
#define   DIR_QCHG_IEN          0x08  // enable Q-channel data has changed Interupt.
#define   DIR_UNLOCK_IEN        0x04  // enable DIR Lock error, AES3 decorder and PLL2 unlocked detected Interrupt
#define   DIR_QCRC_IEN          0x02  // enable Q-Channel sub code CRC error detected Interrupt
#define   DIR_RBTI_IEN          0x01  // enable DIR RA buffer transfer completed Interrupt

#  Receiver Interrupt Mask(Enable) Register #2
#define DIR_INTMASKR2  0x17
#define   DIR_OSLIP_IEN        0x01  // enable DIR Output Data Slip/repeat Error detected Interrupt

#  Receiver Interrupt Mode Register #1
#define DIR_INT_MODER1  0x18
#define   DIR_IM_QCHGM_RE     0x00  // Interruupt at QCHGM Rising Edge
#define   DIR_IM_QCHGM_FE     0x40  // Interruupt at QCHGM Falling Edge
#define   DIR_IM_QCHGM_LS     0x80  // Interruupt at QCHGM Level
#define   DIR_IM_UNLOCKM_RE   0x00  // Interruupt at UNLOCKM Rising Edge
#define   DIR_IM_UNLOCKM_FE   0x10  // Interruupt at UNLOCKM Falling Edge
#define   DIR_IM_UNLOCKM_LS   0x20  // Interruupt at UNLOCKM Level
#define   DIR_IM_QCRCM_RE     0x00  // Interruupt at QCRCM Rising Edge
#define   DIR_IM_QCRCM_FE     0x04  // Interruupt at QCRCM Falling Edge
#define   DIR_IM_QCRCM_LS     0x08  // Interruupt at QCRCM Level
#define   DIR_IM_RBTIM_RE     0x00  // Interruupt at RBTIM Rising Edge
#define   DIR_IM_RBTIM_FE     0x01  // Interruupt at RBTIM Falling Edge
#define   DIR_IM_RBTIM_LS     0x02  // Interruupt at RBTIM Level

#  Receiver Interrupt Mode Register #2
#define DIR_INT_MODER2  0x19
#define   DIR_IM_CSCRCM_RE    0x00  // Interruupt at CSCRCM Rising Edge
#define   DIR_IM_CSCRCM_FE    0x40  // Interruupt at CSCRCM Falling Edge
#define   DIR_IM_CSCRCM_LS    0x80  // Interruupt at CSCRCM Level
#define   DIR_IM_PARITYM_RE   0x00  // Interruupt at PARITYM Rising Edge
#define   DIR_IM_PARITYM_FE   0x10  // Interruupt at PARITYM Falling Edge
#define   DIR_IM_PARITYM_LS   0x20  // Interruupt at PARITYM Level
#define   DIR_IM_VBITM_RE     0x00  // Interruupt at VBITM Rising Edge
#define   DIR_IM_VBITM_FE     0x04  // Interruupt at VBITM Falling Edge
#define   DIR_IM_VBITM_LS     0x08  // Interruupt at VBITM Level
#define   DIR_IM_BPERRM_RE    0x00  // Interruupt at BPERRM Rising Edge
#define   DIR_IM_BPERRM_FE    0x01  // Interruupt at BPERRM Falling Edge
#define   DIR_IM_BPERRM_LS    0x02  // Interruupt at BPERRM Level


#  Receiver Interrupt Mode Register #3
#define DIR_INT_MODER3  0x1a
#define   DIR_IM_OSLIPM_RE    0x00  // Interruupt at OSLIPM Rising Edge
#define   DIR_IM_OSLIPM_FE    0x01  // Interruupt at OSLIPM Falling Edge
#define   DIR_IM_OSLIPM_LS    0x02  // Interruupt at OSLIPM Level

#  GPO Control Register
GPO_CTR1 = 0x1b             # GPO1
SEL_24576 = 0x00            # force Low(default) : select 24.576MHz for 48k,96k,192k...
SEL_225792 = 0x01           # force High : select 22.5792MHz for 44.1k,88.2k,176,4k
#define GPIO_CTR2 0x1c
#define GPIO_CTR3 0x1d
#define GPIO_CTR4 0x1e

#  Q-Channel Sub-Code Data Register (RO)
#define QCH_SCDR1 0x1f
#define QCH_SCDR2 0x20
#define QCH_SCDR3 0x21
#define QCH_SCDR4 0x22
#define QCH_SCDR5 0x23
#define QCH_SCDR6 0x24
#define QCH_SCDR7 0x25
#define QCH_SCDR8 0x26
#define QCH_SCDR9 0x27
#define QCH_SCDR10 0x28

#  Burst Preamble PC Status Regsiter (RO)
#define BP_PC_HBR 0x29
#define BP_PC_LBR 0x2a
#define BP_PD_HBR 0x2b
#define BP_PD_LBR 0x2c

#  SRC Control register #1
SRC_CTR1         =   0x2d
SRC_INPUT_PORTA  =   0x00       # SRC input source is Port A( default )
SRC_INPUT_PORTB  =   0x01       # SRC input source is Port B
SRC_INPUT_DIR    =   0x02       # SRC input source is DIR
SRC_REFCLKS_MCLK =   0x00       # SRC Reference Clock Source is MCLK( default )
SRC_REFCLKS_RXCKI =  0x04       # SRC Reference Clock Source is RXCLKI
SRC_REFCLKS_RXCKO =  0x08       # SRC Reference Clock Source is Recovered Clock from DIR
SRC_OUT_MUTE     =   0x10       # SRC Output Mute enable.
SRC_LRTRACK_EN   =   0x40       # SRC Output Attenuation L/R tracking Enabled

#  SRC Control register #2
SRC_CTR2         =   0x2e
SRC_IGRP_64      =   0x00       # SRC Interpolation Filter Group delay 64 samples( default )
SRC_IGRP_32      =   0x01       # SRC Interpolation Filter Group delay 32 samples
SRC_IGRP_16      =   0x02       # SRC Interpolation Filter Group delay 16 samples
SRC_IGRP_08      =   0x03       # SRC Interpolation Filter Group delay 8 samples
SRC_DDOWN_SMP    =   0x04       # SRC Direct Down Sampling Enable, don't use Decimation Filter
SRC_DEF_DISABLE  =   0x00       # SRC De-Emphasis Disabled ( default )
SRC_DEF_EN48     =   0x08       # SRC De-Emphasis Enabled for fs=48kHz
SRC_DEF_EN441    =   0x10       # SRC De-Emphasis Enabled for fs=44.1kHz
SRC_DEF_EN32     =   0x18       # SRC De-Emphasis Enabled for fs=32kHz
SRC_AUTODEF_EN   =   0x20       # enable SRC Automatic De-Emphasis Function

#  SRC Control Register #3
SRC_CTR3         =   0x2f
SRC_OUT_WL24     =   0x00       # SRC Output data word length is 24bit ( default )
SRC_OUT_WL20     =   0x40       # SRC Output data word length is 20bit
SRC_OUT_WL18     =   0x80       # SRC Output data word length is 18bit
SRC_OUT_WL16     =   0xc0       # SRC Output data word length is 16bit

#  SRC Control Register #4 ( L ch. Attenuation )
SRC_CTR4_LATN    =   0x30

#  SRC Control Register #5 ( R ch. Attenuation )
SRC_CTR5_RATN    =   0x31

#  SRC Ratio Readback Register (RO)
#define SRC_RTO_READBACKR1  0x32
#define SRC_RTO_READBACKR2  0x33

#  SRC Page Selection Register
SRC_PAGE_SELECTR    = 0x7f
CNTR_AND_STATUS_REG = 0x00    # Select Control and Status register page ( default )
DIR_CSTS_UD_BUF     = 0x01    # Select DIR Channel Status buffer and User Data buffer
DIT_CSTS_UD_BUF     = 0x02    # Select DIT Channel Status buffer and User Data buffer



# Write 1 Byte data to SRC4392 register

def write_data_to_SRC4392( regadrs, wrdata ):
    bus.write_byte_data( devadrs, regadrs, wrdata)
    return

# Read out 1 byte from SRC4392 register
def read_data_from_SRC4392( regadrs ):
    value = bus.read_byte_data( devadrs, regadrs )
    return value

# Read out block data with i2c
def blockread(regadrs, value):
    value = bus.read_i2c_block_data( devadrs, regadrs, value)
    return value

#   Set Page Selector Resister
def Set_page_select_reg( page ):
    regadrs = SRC_PAGE_SELECTR
    write_data_to_SRC4392( regadrs, page )
    return

#   Select X-tal OSC for Master Clock source
def Select_MCLK_source( fs ):
    regadrs = GPO_CTR1

    fspm = ( fs % 48 )
    if fspm == 0:
        wrdata = 0x00            #select 24.576MHz
    else:
        wrdata = 0x01            #select 22.5792MHz

    write_data_to_SRC4392( regadrs, wrdata )
    return

#  Setup PortA (I2S input port )
def Set_PortA( mode ):
    regadrs = I2S_PORTA_CTR1

    if mode == 0:
        wrdata =  ( I2SPA_MUTE | I2SPA_IS_LPBACK | I2SPA_SLV_MODE | I2SPA_FMT_16RJ )
    else:
        wrdata =  ( I2SPA_MUTE | I2SPA_IS_LPBACK | I2SPA_SLV_MODE | I2SPA_FMT_I2S_P24 )

    write_data_to_SRC4392( regadrs, wrdata )

    regadrs = I2S_PORTA_CTR2
    wrdata =  ( I2SPA_MCKS_MCLK | I2SPA_MCLK_512FS )
    write_data_to_SRC4392( regadrs, wrdata )

    return

#  Setup PortB (I2S output port )
def Set_PortB( fs ):
    regadrs = I2S_PORTB_CTR1
    wrdata =  ( I2SPB_IS_SRC | I2SPB_MST_MODE | I2SPB_FMT_I2S_P24 )
    write_data_to_SRC4392( regadrs, wrdata )

#     set Sampling Frequency bit

    if fs == 192:
        fs_wrdata = I2SPB_MCLK_128FS

    elif fs == 176:
        fs_wrdata = I2SPB_MCLK_128FS

    elif fs == 96:
        fs_wrdata = I2SPB_MCLK_256FS

    elif fs == 88:
        fs_wrdata  = I2SPB_MCLK_256FS

    elif fs == 48:
        fs_wrdata  = I2SPB_MCLK_512FS

    else:                               # fs=44 or other
        fs_wrdata  = I2SPB_MCLK_512FS


    regadrs = I2S_PORTB_CTR2
    wrdata =  ( I2SPB_MCKS_MCLK | fs_wrdata )
    write_data_to_SRC4392( regadrs, wrdata )

    return


#  Setup DIT ( S/PDIF transmitter )
def Set_DIT( fs ):

#     set Sampling Frequency bit

    if fs == 192:
        fs_wrdata = DIT_MCLK_128FS

    elif fs == 176:
        fs_wrdata = DIT_MCLK_128FS

    elif fs == 96:
        fs_wrdata = DIT_MCLK_256FS

    elif fs == 88:
        fs_wrdata  = DIT_MCLK_256FS

    elif fs == 48:
        fs_wrdata  = DIT_MCLK_512FS

    else:                               # fs=44 or other
        fs_wrdata  = DIT_MCLK_512FS

    regadrs = DIT_CTR1
    wrdata =  (( DIT_TXCLK_MCLK | DIT_IS_SRC ) | fs_wrdata )
    write_data_to_SRC4392( regadrs, wrdata )

    regadrs = DIT_CTR2
    wrdata =  0x00
    write_data_to_SRC4392( regadrs, wrdata )

    regadrs = DIT_CTR3
    wrdata =  DIT_TXCUSTS_HIF          #0x01
    write_data_to_SRC4392( regadrs, wrdata )

    return

#  Set up Sample Rate Converter in SRC4392
def Set_SRC( mode ):
    regadrs = SRC_CTR1
    wrdata =  ( SRC_INPUT_PORTA | SRC_REFCLKS_MCLK )
    write_data_to_SRC4392( regadrs, wrdata )

    regadrs = SRC_CTR2
    wrdata =  0x01
    write_data_to_SRC4392( regadrs, wrdata )

    regadrs = SRC_CTR3
    wrdata =  0x00
    write_data_to_SRC4392( regadrs, wrdata )

    regadrs = SRC_CTR4_LATN
    wrdata =  0x00
    write_data_to_SRC4392( regadrs, wrdata )

    regadrs = SRC_CTR5_RATN
    wrdata =  0x00
    write_data_to_SRC4392( regadrs, wrdata )

    return

"""
       Set SRC S/PDIF Transmitter Channel Status

         input:   fs
                       44: 44.1kHz
                       48: 48kHz
                       88: 88.2kHz
                       96: 96  kHz
                      176: 176.4kHz
                      192: 192kHz
"""

def Set_txchsts( fs ):

#      Allow to update through I2C

    regadrs =  DIT_CTR2;
    wrdata  = DIT_TXBTD_DA;
    write_data_to_SRC4392( regadrs, wrdata )

#     Select Register Page 2

    regadrs =  SRC_PAGE_SELECTR;
    wrdata  =  DIT_CSTS_UD_BUF;
    write_data_to_SRC4392( regadrs, wrdata );


#     set Sampling Frequency bit

    if fs == 192:
        Lfs_wrdata = 0x70               # b24,b25,b26,b27  0,1,1,1

    elif fs == 176:
        Lfs_wrdata = 0x30               # b24,b25,b26,b27  0,0,1,1

    elif fs == 96:
        Lfs_wrdata = 0x50               # b24,b25,b26,b27  0,1,0,1

    elif fs == 88:
        Lfs_wrdata  = 0x10              # b24,b25,b26,b27  0,0,0,1

    elif fs == 48:
        Lfs_wrdata  = 0x40              # b24,b25,b26,b27  0,1,0,0

    else:                               # fs=44 or other
        Lfs_wrdata  = 0x00              # b24,b25,b26,b27  0,0,0,0

    Rfs_wrdata = Lfs_wrdata                          # copy to R ch. status
    Lwl_wrdata = (( ~Lfs_wrdata >> 4 ) | 0xa0 )      # set b36, b37, b38, b39 and Word length
    Rwl_wrdata = Lwl_wrdata                          # copy to R ch. status


#   Configure Transmitter Ch.status

    regadrs =  6
    wrdata  =  Lfs_wrdata
    write_data_to_SRC4392( regadrs, wrdata );

    regadrs =  7
    wrdata  =  Rfs_wrdata
    write_data_to_SRC4392( regadrs, wrdata );

    regadrs =  8
    wrdata  =  Lwl_wrdata
    write_data_to_SRC4392( regadrs, wrdata );

    regadrs =  9
    wrdata  =  Rwl_wrdata
    write_data_to_SRC4392( regadrs, wrdata );

#   Reset to Register Page 0

    regadrs =  SRC_PAGE_SELECTR;
    wrdata  =  CNTR_AND_STATUS_REG;

    write_data_to_SRC4392( regadrs, wrdata )

#   Allow to transfer updated status to TX buffer

    regadrs =  DIT_CTR2;
    wrdata  = DIT_TXBTD_EN;

    write_data_to_SRC4392( regadrs, wrdata )

    return

    
#   Init SRC4392
#           Release Power-Down state, set Reference Clock and MCLK source, 
#           configure Output port and configure default Input port.
#           Port A: I2S(LPCM) input
#           Port B: I2S output
#           Transmitter : S/PDIF output

def init_SRC4392():

    page = CNTR_AND_STATUS_REG
    Set_page_select_reg( page )

#   Relaese SRC4392 from Power-Down state

    regadrs = PDWN_RST_CTR;
    wrdata  = ( RELEASE_RESET_SRC | RELEASE_PDALL | RELEASE_PDPA | RELEASE_PDPB | RELEASE_PDTX | RELEASE_PDRX | RELEASE_PDSRC );

    write_data_to_SRC4392( regadrs, wrdata )

#   Reset SRC after wake up

    wrdata  = ( RESET_SRC | RELEASE_PDALL | RELEASE_PDPA | RELEASE_PDPB | RELEASE_PDTX | RELEASE_PDRX | RELEASE_PDSRC );

    write_data_to_SRC4392( regadrs, wrdata )

    time.sleep( 1 )            #wait for stable clock.
    
#      Release Reset

    wrdata  = ( RELEASE_RESET_SRC | RELEASE_PDALL | RELEASE_PDPA | RELEASE_PDPB | RELEASE_PDTX | RELEASE_PDRX | RELEASE_PDSRC );
    write_data_to_SRC4392( regadrs, wrdata )


#      set X-tal OSC to default MCLK 24.576Mhz

    #fs= 96
    Select_MCLK_source( fs )
  
#      Configure Port-A as SRC input port for Raspberry pi
#       I2S Port A control register #2 : Set input mode for Raspberry pi, set SDOUTA disable,
#       I2S slave mode and I2S 24bit Phillips format.  set Clock source and rate for Port A input

    # mode = 0          #  16bit Right Justify mode
    mode = 1            #  24bit I2S mode
    Set_PortA( mode )

#      Configure Port-B as SRC output port
#       I2S Port B control register #2 : Release Mute, Input source is SRC, I2S master mode and I2S 24bit Phillips format
#                set Clock source and rate for Port B output.   Reference Clock = 128fs -> 192kHz or 176.4kHz

    mode = 0
    #fs = 96
    Set_PortB( fs )

#      Init Transmitter 
#         Configure Transmitter Control register 1 and Control register 2

    #fs = 96
    Set_DIT( fs )


#    RAL-SRC4392 does not use DIR Receiver

#    Configure SRC Control Register #1, #2, #3, #4, #5

#   Added output Mute control on 06-Sept.-2011

    mode = 0
    Set_SRC( mode )

#     Select Register Page 2 and init. Transmitter Ch. Status buffer

    page = DIT_CSTS_UD_BUF
    Set_page_select_reg( page )

    wrdata = 00;

    for regadrs in range(12):
        write_data_to_SRC4392( regadrs, wrdata )

#   Reset to Register Page 0

    page = CNTR_AND_STATUS_REG
    Set_page_select_reg( page )

    # fs = 96  #44
    Set_txchsts( fs )

if __name__ == '__main__':
    print "RAL-SRC4392Pi initialize start V.1R0 01/Sept./2017"

# added argument for src4392_2R7.py  10/Oct./2017

    from sys import argv
    args = sys.argv

    if len(args) >= 2:
        fs1 = args[1]
        fs2 = float( fs1 )
        fs  = int(fs2)            # set up 44, 88 and 176
    else:
        fs = 96                   # default S/PDIF output fs

    init_SRC4392()
